package com.sofeh.devicestate;

import java.util.Comparator;
import java.util.List;

import android.net.Uri;
import android.os.Bundle;
import android.provider.Settings;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.text.format.DateUtils;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;

public class ApplicationActivity extends Activity {

	ListView listView;
	
	PackageManager packageManager;
	List<PackageInfo> packages;
	ArrayAdapter<PackageInfo> adapter;
	
	String lineSep = System.getProperty("line.separator");
	Boolean extraInfo = false;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_application);
		
		listView = (ListView) findViewById(R.id.listView1);
		
		final PackageManager packageManager = getPackageManager();
		
		final List<PackageInfo> packages = packageManager.getInstalledPackages(0/*PackageManager.GET_META_DATA*/);

	    adapter = new ArrayAdapter<PackageInfo>(this, R.layout.list_layout, R.id.text1, packages) 
	    {
			@Override
	    	public View getView(int position, View convertView, ViewGroup parent) {
	    	  View view = super.getView(position, convertView, parent);
	    	  TextView text1 = (TextView) view.findViewById(R.id.text1);
	    	  TextView text2 = (TextView) view.findViewById(R.id.text2);
	    	  TextView text3 = (TextView) view.findViewById(R.id.text3);
	    	  ImageView imageView1 = (ImageView) view.findViewById(R.id.imageView1);
	    	  view.findViewById(R.id.imageView2).setVisibility(View.VISIBLE);
	    	  
	    	  PackageInfo pac = packages.get(position);
	    	  ApplicationInfo app = pac.applicationInfo;
	    	  
	    	  text1.setText(packageManager.getApplicationLabel(app));
	    	  imageView1.setImageDrawable(packageManager.getApplicationIcon(app));
	    	  
	    	  String str;
    		  if ((app.flags & ApplicationInfo.FLAG_SYSTEM) != 0)
    		  {
    			  text1.setTextColor(Color.rgb(150,0,0));
       			  str = "System application";
    		  }
	    	  else
	    	  {  
    			  text1.setTextColor(Color.rgb(0,150,0));
    			  str = "Downloaded application";
	    	  }
	    		  
    		  if ((app.flags & ApplicationInfo.FLAG_EXTERNAL_STORAGE) != 0)
       			  str = str+lineSep+"Installed on SD card";
    		  else	  
       			  str = str+lineSep+"Installed on device";
	  
    		  text2.setText(str);
	    	  
    		  if (extraInfo)
    			  text3.setText("Version: "+pac.versionName+" ("+pac.versionCode+")"+lineSep+
    					  		"First Install Time: "+lineSep+
    					  		"  "+DateUtils.formatDateTime(ApplicationActivity.this, pac.firstInstallTime, DateUtils.FORMAT_SHOW_DATE | DateUtils.FORMAT_SHOW_TIME | DateUtils.FORMAT_SHOW_YEAR)+lineSep+
    					  		"Last Update Time: "+lineSep+
    					  		"  "+DateUtils.formatDateTime(ApplicationActivity.this, pac.lastUpdateTime, DateUtils.FORMAT_SHOW_DATE | DateUtils.FORMAT_SHOW_TIME | DateUtils.FORMAT_SHOW_YEAR)+lineSep+
    					  		lineSep+
    					  		//"non-Localized Label: "+app.nonLocalizedLabel+lineSep+
    					  		"Package Name: "+app.packageName+lineSep+
    					  		"Process Name: "+app.processName+lineSep+
    					  		"Class Name: "+app.className+lineSep+
    					  		//"App Name: "+app.name+lineSep+
    					  		//"Manage Space Activity Name: "+app.manageSpaceActivityName+lineSep+
    					  		//"Backup Agent Name: "+app.backupAgentName+lineSep+
    					  		"Task Affinity: "+app.taskAffinity+lineSep+
    					  		"Target SDK Version: "+app.targetSdkVersion+lineSep+
    					  		lineSep+
    					  		"Source Dir: "+lineSep+"  "+app.sourceDir+lineSep+
    					  		"Public Source Dir:"+lineSep+"  "+app.publicSourceDir+lineSep+
    					  		"Data Dir:"+lineSep+"  "+app.dataDir//+lineSep+
    					  		//"Native Library Dir: "+app.nativeLibraryDir+lineSep+
    					  		//"Shared Library Files: "+app.sharedLibraryFiles+lineSep+
    					  		//"Permissions: "+app.permission
    					  		);
    		  else
    			  text3.setText("v"+pac.versionName);
	    	  
   			  return view;
	    	}
	    };	
	    
	    adapter.sort(new Comparator<PackageInfo>() {
	        @Override
	        public int compare(PackageInfo lhs, PackageInfo rhs) {
	        	String lsys = "0";
	        	String rsys = "0";
	        	if ((lhs.applicationInfo.flags & ApplicationInfo.FLAG_SYSTEM) != 0) lsys = "1";
	        	if ((rhs.applicationInfo.flags & ApplicationInfo.FLAG_SYSTEM) != 0) rsys = "1";
	        		
        		return (lsys+packageManager.getApplicationLabel(lhs.applicationInfo).toString()).compareTo(rsys+packageManager.getApplicationLabel(rhs.applicationInfo).toString());
	        }
	    });
	    
	    listView.setAdapter(adapter);
        
	    listView.setClickable(true);
	    listView.setOnItemClickListener(new OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view,
                    int position, long id) {

  	    	  	final PackageInfo pac = packages.get(position);
  	    	  	String[] items;
				if ((pac.applicationInfo.flags & ApplicationInfo.FLAG_SYSTEM) != 0)
  	    	  		items = new String[] {"Launch", "Share", "Manage"};
				else
  	    	  		items = new String[] {"Launch", "Share", "Manage", "Uninstall"};
  	    	  	
    			new AlertDialog.Builder(ApplicationActivity.this)
      	    	.setTitle(packageManager.getApplicationLabel(pac.applicationInfo))
      	    	.setNegativeButton("cancel", null)
      	    	.setItems(items, new DialogInterface.OnClickListener() {
      	    		public void onClick(DialogInterface dialog, int which) {
      	    			Intent intent;
      	    			switch (which) {
      	    				case 0:
   	    						intent = packageManager.getLaunchIntentForPackage(pac.packageName);
       	      	    	    	if (intent != null) startActivity(intent);
      	    					break;
      	    					
      	    				case 1 :
      	    					intent = new Intent(Intent.ACTION_SEND); 
      	    				    intent.setType("application/vnd.android.package-archive");
      	    				    intent.putExtra(Intent.EXTRA_STREAM, Uri.parse("file://"+pac.applicationInfo.sourceDir));
      	    				    startActivity(Intent.createChooser(intent, "Share via"));
      	    					break;
      	    				
      	    				case 2 :	
      	      	    	    	intent = new Intent();
      	      	    	    	intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
      	      	    	        intent.setData(Uri.fromParts("package", pac.packageName, null));
      	      	            	startActivityForResult(intent, 1000);
      	    					break;
      	    					
      	    				case 3 :	
      	      	            	/* install
      	      	            	Intent intent = new Intent(Intent.ACTION_VIEW);
      	      	            	intent.setDataAndType(apkUri, "application/vnd.android.package-archive");
      	      	            	startActivity(intent);
      	      	            	*/
      	      	            	startActivityForResult(new Intent(Intent.ACTION_DELETE, Uri.fromParts("package", pac.packageName,null)), 1000);
      	    					break;
						}
      	    		}
      	    	})
      	    	.show();
  	    	  	
            }
        });		
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
	    switch (requestCode) {
	        case 1000:
        		adapter.notifyDataSetChanged(); // Refresh List
	        	break;
	    }
	    super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.application, menu);
		
		return true;
	}

    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
    	//check selected menu item
    	switch (item.getItemId()) {
			case R.id.action_extraInfo :
				extraInfo = ! extraInfo;
        		adapter.notifyDataSetChanged(); // Refresh List
    			break;
    		case R.id.action_back :
    			this.finish();
    			break;
    	}
		return true;
    }
}
