package com.sofeh.devicestate;

import java.util.List;

import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.app.Activity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

public class SensorActivity extends Activity implements SensorEventListener {
	
	@SuppressWarnings("deprecation")
	private String[] sensorTypeToString(Sensor sensor) {
		String[] result = new String[]{"Unknown", "unknown", "", ""};
		int drawable = R.drawable.sensor_unknown;
		
	    switch (sensor.getType()) {
    		case Sensor.TYPE_ACCELEROMETER : 
    			result[0] = "Accelerometer";
    			result[1] = "m/s2";
    			result[2] = "Meter per Square Second";
    			drawable = R.drawable.sensor_orientation;
    			break;
    		case Sensor.TYPE_AMBIENT_TEMPERATURE : 
				result[0] = "Ambient Temperature";
     			result[1] = "C";
     			result[2] = "degree Celsius";
     			drawable = R.drawable.sensor_temperature;
				break;
    		case Sensor.TYPE_GAME_ROTATION_VECTOR : 
    			result[0] = "Game Rotation Vector";
    			result[1] = "";
    			result[2] = "degree";
     			drawable = R.drawable.sensor_orientation;
    			break;
    		case Sensor.TYPE_GEOMAGNETIC_ROTATION_VECTOR : 
    			result[0] = "Geomagnetic Rotation Vector";
    			result[1] = "";
    			result[2] = "degree";
     			drawable = R.drawable.sensor_orientation;
    			break;
    		case Sensor.TYPE_GRAVITY : 
    			result[0] = "Gravity";
    			result[1] = "m/s2";
    			result[2] = "Meter per Square Second";
    			drawable = R.drawable.sensor_gravity;
    			break;
    		case Sensor.TYPE_GYROSCOPE : 
    			result[0] = "Gyroscope";
    			result[1] = "rad/s";
    			result[2] = "Radian per second";
    			drawable = R.drawable.sensor_orientation;
    			break;
    		case Sensor.TYPE_GYROSCOPE_UNCALIBRATED : 
    			result[0] = "Gyroscope (Uncalibrated)";
    			result[1] = "rad/s";
    			result[2] = "Radian per second";
    			drawable = R.drawable.sensor_orientation;
    			break;
    		case Sensor.TYPE_LIGHT : 
    			result[0] = "Light";
    			result[1] = "lx";
    			result[2] = "lux";
    			drawable = R.drawable.sensor_light;
    			break;
    		case Sensor.TYPE_LINEAR_ACCELERATION : 
    			result[0] = "Linear Acceleration";
    			result[1] = "m/s2";
    			result[2] = "Meter per Square Second";
    			drawable = R.drawable.sensor_orientation;
    			break;
    		case Sensor.TYPE_MAGNETIC_FIELD : 
    			result[0] = "Magnetic Field";
    			result[1] = "T";
    			result[2] = "micro-Tesla";
    			drawable = R.drawable.sensor_compass;
    			break;
    		case Sensor.TYPE_MAGNETIC_FIELD_UNCALIBRATED : 
    			result[0] = "Magnetic Field (Uncalibrated)";
    			result[1] = "T";
    			result[2] = "micro-Tesla";
    			drawable = R.drawable.sensor_compass;
    			break;
    		case Sensor.TYPE_ORIENTATION : 
    			result[0] = "Orientation";
    			result[1] = "";
    			result[2] = "degree";
    			drawable = R.drawable.sensor_orientation;
    			break;
    		case Sensor.TYPE_PRESSURE : 
    			result[0] = "Pressure";
    			result[1] = "hPa";
    			result[2] = "millibar";
    			drawable = R.drawable.sensor_pressure;
    			//result[1] = "N/m2";
    			//result[2] = "Newton per Square Meter";
    			break;
    		case Sensor.TYPE_PROXIMITY : 
    			result[0] = "Proximity";
    			result[1] = "cm";
    			result[2] = "centimeter";
    			drawable = R.drawable.sensor_proximity;
    			break;
     		case Sensor.TYPE_RELATIVE_HUMIDITY : 
     			result[0] = "Relative Humidity";
     			result[1] = "%";
     			result[2] = "percent";
     			drawable = R.drawable.sensor_humidity;
     			break;
     		case Sensor.TYPE_ROTATION_VECTOR : 
     			result[0] = "Rotation Vector";
    			result[1] = "";
    			result[2] = "degree";
     			drawable = R.drawable.sensor_orientation;
     			break;
     		case Sensor.TYPE_SIGNIFICANT_MOTION : 
     			result[0] = "Significant Motion";
     			result[1] = "";
     			result[2] = "";
     			drawable = R.drawable.sensor_orientation;
				break;
     		case Sensor.TYPE_STEP_COUNTER : 
     			result[0] = "Step Counter";
     			result[1] = "";
     			result[2] = "";
     			drawable = R.drawable.sensor_step;
     			break;
     		case Sensor.TYPE_STEP_DETECTOR : 
     			result[0] = "Step Detector";
     			result[1] = "";
     			result[2] = "";
     			drawable = R.drawable.sensor_step;
     			break;
     		case Sensor.TYPE_TEMPERATURE : 
     			result[0] = "Temperature";
     			result[1] = "C";
     			result[2] = "degree Celsius";
     			drawable = R.drawable.sensor_temperature;
     			break;
	    }	
		result[3] = Integer.toString(drawable);
		return result;
	}

	String[] list = new String[1000000];
	ArrayAdapter<Sensor> adapter; 
	String lineSep = System.getProperty("line.separator");
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_sensor);
		
		ListView listView = (ListView) findViewById(R.id.listView1);
		// Sensor List
		final SensorManager sensorManager = (SensorManager)getSystemService(SENSOR_SERVICE);
		final List<Sensor> sensorList = sensorManager.getSensorList(Sensor.TYPE_ALL);
		
		/* SIMPLE:
		final ArrayList<String> list = new ArrayList<String>();
		for(Sensor sensor: sensorList){
	        list.add(sensor.getName());
	    }
		ArrayAdapter adapter = new ArrayAdapter(this,
	            android.R.layout.simple_list_item_1, list);*/
		
	    adapter = new ArrayAdapter<Sensor>(this, R.layout.list_layout, R.id.text2, sensorList) 
	    {
			@Override
	    	public View getView(int position, View convertView, ViewGroup parent) {
	    	  View view = super.getView(position, convertView, parent);
	    	  TextView text1 = (TextView) view.findViewById(R.id.text1);
	    	  TextView text2 = (TextView) view.findViewById(R.id.text2);
	    	  TextView text3 = (TextView) view.findViewById(R.id.text3);
	    	  ImageView imageView1 = (ImageView) view.findViewById(R.id.imageView1);
	    	  
 	    	  Sensor sensor = sensorList.get(position);
 	    	  
 	    	  String[] sensorType = sensorTypeToString(sensor);
 	    	  imageView1.setImageResource(Integer.parseInt(sensorType[3]));
	    	  text1.setText(sensorType[0]);
	    	  if (sensorManager.getDefaultSensor(sensor.getType()) == sensor) 
	    		  text2.setText("Default "+sensorType[0]+" sensor");
	    	  else
	    		  text2.setText("");
	    	  text3.setText("Name: "+sensor.getName()+lineSep+
  			  		"Vendor: "+sensor.getVendor()+lineSep+
  			  		"Version: "+sensor.getVersion()+lineSep+lineSep+
  			  		"Power: "+sensor.getPower()+" mA (milliampere)"+lineSep+
  			  		"Resolution: "+sensor.getResolution()+" "+sensorType[1]+lineSep+
  			  		"Maximum Range: "+sensor.getMaximumRange()+" "+sensorType[1]+lineSep+lineSep+
  			  		//"  min delay: "+sensor.getMinDelay()+lineSep+
  			  		"Data: "+sensorType[1]+" ("+sensorType[2]+")"+lineSep+list[sensor.getType()]);
	    	  return view;
	    	}
	    };	    
        listView.setAdapter(adapter);
        
	    listView.setClickable(false);
        
		// Sensor Data
        //SINGLE: sensorManager.registerListener((SensorEventListener) this, sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER), SensorManager.SENSOR_DELAY_NORMAL);
		for (Sensor sensor: sensorList)
	        sensorManager.registerListener((SensorEventListener) this, sensor, SensorManager.SENSOR_DELAY_NORMAL);

	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.sensor, menu);
		return true;
	}
	
    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
    	//check selected menu item
    	switch (item.getItemId()) {
    		case R.id.action_back :
    			this.finish();
    			break;
    	}
		return true;
    }	
    		
	@Override
	public void onAccuracyChanged(Sensor arg0, int arg1) {
	}
	
	@Override
	public void onSensorChanged(SensorEvent event) {
		if (event.sensor.getType() < list.length) {
			list[event.sensor.getType()] = "";
			for (float value : event.values) {
				list[event.sensor.getType()] = list[event.sensor.getType()] + "    "+Float.toString(value)+lineSep;
			};
			adapter.notifyDataSetChanged(); // Refresh List
		}
	}
}
